
*
{
margin: 0px;
padding: 0px;
}

a:link {
	color: #4DA0C6;
}

a:hover {
	text-decoration: underline;
	color: #FF0000;
}

a:visited {
	color: #4DA0C6;
}

textarea {
	width:762px;
}

body {
	background: #0271BF;
	font: 13px Arial, Helvetica, sans-serif;
	color: #000000;
}

#page
{
	margin: 20px auto;
	width: 847px;        
}

#logo{
	margin-left:60px;
}

#header{
    height: 92px;
    padding-top: 18px;
    width: 847px;
    background: url(images/head.png) no-repeat;
}

.block_header{    
    text-align: left;
    padding-left: 25px;
    height: 30px;
}

#header .text{
    font-weight: bold;
    font-size: 14px;
    padding-top: 6px;
    margin-left: 5px;
    float: left;
}

#header .info{
    font-weight: bold;
    font-size: 14px;
    padding-top: 6px;
    float: left;
    width: 700px;
    text-align: center;
}

.wmenu{
    
}

.wmenu ul{
    list-style:none;
    margin:0;
    padding:0;
    margin-left:20px;
    height: 35px;
}

.space{
    margin-left: 18px;
}

.wmenu li{
    float:left;
    background: url(images/sep.png) left center no-repeat;
}

.wmenu li .title_software{
    float:left;
    background: url(images/software.png) left center no-repeat;
    font-size:14px;
    font-weight:bold;
    color:#0271BF;
    padding:8px;
    padding-left: 20px;
    padding-right: 5px;
    margin-left:4px;
    height: 20px;
    text-shadow:1px 1px 0 #F8F8F8;
}

.wmenu li .title_instance{
    float:left;
    background: url(images/settings2.png) left center no-repeat;
    font-size:14px;
    font-weight:bold;
    color:#0271BF;
    padding:8px;
    padding-left: 20px;
    padding-righclass="title_instance"t: 5px;
    height: 20px;
    margin-left:2px;
    text-shadow:1px 1px 0 #F8F8F8;
}


.wmenu ul li a{
    display:block;    
    height: 20px;
    Color:#000;
    font-size:14px;
    text-decoration:none;
    padding:8px;
    padding-left: 6px;
}

.wmenu ul li a:focus, .wmenu ul li a:hover, .wmenu ul li a.active{
    color:#0271BF;
    background:url(images/menu_hover.png) repeat-x;
}

#main
{
    min-height: 400px;
    background: url(images/main_bg.png) repeat-y;
    padding:10px;
    padding-left: 40px;
    padding-right: 40px;
    width: 767px;
}

#main h2{
	margin-top:10px;
	font-size:14px;
	font-weight:bold;
	text-decoration: capitalize;
	color:#0271BF;
}

#footer{
    height: 40px;
    width: 767px;
    background: url(images/footer.png) no-repeat;
    padding-left: 40px;
    padding-right: 40px;
    padding-top: 25px;
    text-align: center;
}

.button {
	padding: 5px 10px;
	display: inline;
	background: #eee;
	border: none;
	color: #000;
	cursor: pointer;
	font-weight: bold;
	border:1px solid #000;
	margin-top:10px;
}
.button:hover {
	background-position: 0 center;
	background: #666666;
	color: #fff;
}
.button:active {
	background-position: 0 top;
	position: relative;
	top: 1px;
	padding: 6px 10px 4px;
}

.flash{		
	margin-left:20px;
	color:#FF5500;
	font-style:italic;
	font-size:14px;
}

.flashes{
	margin-bottom:10px;
}

.message {
	color:#FF5500;
	line-height:21px;
}

.clear{
    clear: both;
}
