{% extends "layout.html" %}
{% block body %}
<h2>Instance inspection</h2><br/>
<h2>Supervisor:</h2>
<textarea cols=103 rows=10 readonly>{{ supervisor }}</textarea><br>
<h2>SLAP:</h2>
{% for item in slap_status %}
<b>{{ item[0 ]}}</b><br>
<textarea cols=103 rows=5 readonly>{{ item[1] }}</textarea><br>
{% endfor %}
<h2>File content:</h2>
<textarea cols=103 rows=40 readonly>{{ file_content }}</textarea><br>
{% endblock %}
